@@ -47,7 +47,7 @@ _logger = logging.getLogger(__name__)
 
 
 class HazelcastClient(object):
-    """Hazelcast client instance to access access and manipulate
+    """Hazelcast client instance to access and manipulate
     distributed data structures on the Hazelcast clusters.
 
     Keyword Args:
@@ -327,6 +327,11 @@ class HazelcastClient(object):
             :class:`hazelcast.errors.IndeterminateOperationStateError`. However,
             even if the invocation fails, there will not be any rollback on other
             successful replicas. By default, set to ``False`` (do not fail).
+        creds_username (str): Username for credentials authentication (Enterprise feature).
+        creds_password (str): Password for credentials authentication (Enterprise feature).
+        token_provider (hazelcast.security.TokenProvider): Token provider for
+            custom authentication (Enterprise feature). Note that token_provider
+            setting has priority over credentials settings.
     """
 
     _CLIENT_ID = AtomicInteger()
